#include "stm32f4_discovery.h"
#include "stm32f4xx_conf.h"
#include "mac.h"
#include "adb.h"

__IO uint8_t mackey[kBuffSize];
__IO uint8_t keyindex;
__IO uint8_t key2index;
__IO uint8_t keypad;

__IO uint16_t mousex;
__IO uint16_t mousey;
__IO uint16_t mousebits;

__IO struct mackbd mackbd;

extern unsigned long tenustimer;
extern uint8_t modifierkeys;


void handle_mouseaxis(__IO uint16_t *movement, uint32_t pinone, uint32_t pintwo) {
	static uint8_t counter = 100;

	if(*movement & 0x00FF) {
		uint8_t movecount = *movement & 0x00FF;

		if(counter--) return;
		counter = 100;

		*movement = (*movement & 0xFF00) | (movecount-1);
		//if((*movement & 0x00FF) < 2) return;

		if(!(*movement & 0x0100)) {
			if(mousebits & pinone) {
				GPIO_ResetBits(GPIOD, pintwo);
				GPIO_ResetBits(GPIOD, pinone);
				mousebits &= ~(pinone | pintwo);
			} else {
				GPIO_SetBits(GPIOD, pintwo);
				GPIO_SetBits(GPIOD, pinone);
				mousebits |= pinone | pintwo;
			}
		} else {
			if(mousebits & pinone) {
				GPIO_SetBits(GPIOD, pintwo);
				mousebits |= pintwo;
				GPIO_ResetBits(GPIOD, pinone);
				mousebits &= ~pinone;
			} else {
				GPIO_ResetBits(GPIOD, pintwo);
				mousebits &= ~pintwo;
				GPIO_SetBits(GPIOD, pinone);
				mousebits |= pinone;
			}
		}
	}
	return;
}

void MacHandler(void) {
	GPIO_InitTypeDef GPIO_InitStructure;
	static uint8_t respbit;

	handle_mouseaxis(&mousex, kMSEX1, kMSEX2);
	handle_mouseaxis(&mousey, kMSEY1, kMSEY2);
	switch(mackbd.state) {
	case kMacKbdStateIdle:
		if(GPIO_ReadInputDataBit(GPIOD, kMacKbdDataPin) == 0) {
			mackbd.state = kMacKbdStateReadSettle;
			mackbd.delay = 18;
			mackbd.bits = 0;
			mackbd.cmd = 0;
			mackbd.clockstate = 3;
		}
		break;
	case kMacKbdStateReadSettle:
		if(--mackbd.delay == 0) {
			mackbd.state = kMacKbdStateReading;
		}
		break;
	case kMacKbdStateReading:
		switch(mackbd.clockstate) {
		case 0:
			if(--mackbd.delay == 0) {
				mackbd.delay = 8;
				mackbd.clockstate = 4;
				GPIO_SetBits(GPIOD, kMacKbdClockPin);
			}
			break;
		case 1:
			if(--mackbd.delay == 0) {
				if(mackbd.bits == 8) {
					// Done clocking in all the bits
					mackbd.state = kMacKbdStateIdle;
					HandleMacCommand();
				} else {
					// Not done, transition to next clock
					mackbd.delay = 18;
					mackbd.clockstate = 0;
					GPIO_ResetBits(GPIOD, kMacKbdClockPin);
				}
			}
			break;
		case 3:
			// We just entered the read state, bring the clock low
			mackbd.clockstate = 0;
			mackbd.delay = 18;
			GPIO_ResetBits(GPIOD, kMacKbdClockPin);
			break;
		case 4: /* read the data 80uS into the rising clock */
			if(--mackbd.delay == 0) {
				mackbd.cmd |= GPIO_ReadInputDataBit(GPIOD, kMacKbdDataPin) << (7 - mackbd.bits++);
				mackbd.delay = 14;
				mackbd.clockstate = 1;
			}
			break;
		default:
			break;
		};
		break;
	case kMacKbdStateWriteSettle:
		if(--mackbd.delay == 0) {
			if(GPIO_ReadInputDataBit(GPIOD, kMacKbdDataPin) == 0) {
				/* "... the computer then indicates that it is ready 
				 * to receive the keyboard's response by setting the 
				 * Keyboard Data line high
				 */
				mackbd.delay = 1;
			} else {
				mackbd.state = kMacKbdStateWriting;
				mackbd.clockstate = 3;
				mackbd.bits = 0;
			}
		}
		break;
	case kMacKbdStateWriting:
		switch(mackbd.clockstate) {
		case 0:
			if(--mackbd.delay == 0) {
				mackbd.delay = 18;
				mackbd.clockstate = 1;
				GPIO_SetBits(GPIOD, kMacKbdClockPin);
			}
			break;
		case 1:
			if(--mackbd.delay == 0) {
				if(mackbd.bits == 8) {
					mackbd.state = kMacKbdStateIdle;
					GPIO_InitStructure.GPIO_Pin = kMacKbdDataPin;
					GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
					GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
					GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
					GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
					GPIO_Init(GPIOD, &GPIO_InitStructure);
				} else {
					respbit = (mackbd.response >> (7-mackbd.bits)) & 0x01;
					mackbd.delay = 16;
					mackbd.clockstate = 0;
					GPIO_ResetBits(GPIOD, kMacKbdClockPin);
					respbit ? GPIO_SetBits(GPIOD, kMacKbdDataPin) : GPIO_ResetBits(GPIOD, kMacKbdDataPin);
					mackbd.bits++;
				}
			}
			break;
		case 3:
			GPIO_InitStructure.GPIO_Pin = kMacKbdDataPin;
			GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
			GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
			GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
			GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
			GPIO_Init(GPIOD, &GPIO_InitStructure);

			respbit = (mackbd.response >> (7-mackbd.bits)) & 0x01;
			mackbd.bits++;
			mackbd.clockstate = 0;
			mackbd.delay = 16;
			GPIO_ResetBits(GPIOD, kMacKbdClockPin);
			respbit ? GPIO_SetBits(GPIOD, kMacKbdDataPin) : GPIO_ResetBits(GPIOD, kMacKbdDataPin);
			break;
			
		default:
			break;
		};
		break;
	case kMacKbdStateInstantWait:
	case kMacKbdStateInquiryWait:
		if(mackey[keyindex]) {
			mackbd.state = kMacKbdStateWriteSettle;
			mackbd.delay = 1;
			mackbd.response = mackey[keyindex];
			mackey[keyindex] = 0;
			keyindex = (keyindex + 1) % kBuffSize;
		} else if(--mackbd.delay == 0) {
			// If no keyboard press, send a null
			mackbd.state = kMacKbdStateWriteSettle;
			mackbd.delay = 1;
			mackbd.response = 0x7B;
		}
		break;
	case kMacKbdStateTest:
		mackbd.state = kMacKbdStateWriteSettle;
		mackbd.delay = 1;
		mackbd.response = 0x7D;
		break;
	default:
		break;
	};
}

void MacInit(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	GPIO_InitTypeDef GPIO2_struct;
	int i;

	mousex = 0;
	mousey = 0;
	tenustimer = 0;
	keyindex = 0;
	key2index = 0;
	keypad = 0;
	for(i = 0; i < kBuffSize; i++) mackey[i] = 0;

	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);

	// Mouse

	GPIO2_struct.GPIO_Pin = kMSEX1 | kMSEX2 | kMSEY1 | kMSEY2;
	GPIO2_struct.GPIO_Mode = GPIO_Mode_OUT;
	GPIO2_struct.GPIO_OType = GPIO_OType_PP;
	GPIO2_struct.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO2_struct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOD, &GPIO2_struct);

	// Mac Keyboard
	// Clock
	GPIO_InitStructure.GPIO_Pin = kMacKbdClockPin;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOD, &GPIO_InitStructure);

	// Data
	GPIO_InitStructure.GPIO_Pin = kMacKbdDataPin;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_Init(GPIOD, &GPIO_InitStructure);

	// Mouse button
	GPIO_InitStructure.GPIO_Pin = kMSESW;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_Init(GPIOD, &GPIO_InitStructure);

	GPIO_SetBits(GPIOD, kMacKbdClockPin);

	mousebits = 0;
	GPIO_SetBits(GPIOD, kMSEX1 | kMSEY1);
	mousebits |= (kMSEX1 | kMSEY1);
	GPIO_ResetBits(GPIOD, kMSEX2 | kMSEY2);
	mousebits &= ~(kMSEX2 | kMSEY2);

	GPIO_SetBits(GPIOD, kMSESW);

	mackbd.state = kMacKbdStateIdle;
}

uint8_t PS2ToMacScancode(uint8_t code, uint8_t keyup, uint8_t extended)
{
	uint8_t maccode = 0;
	uint8_t insertshift = 0;
	keypad = 0;
	if(!extended) {
		switch(code) {
		case 0x1C: maccode = 0x01; break; // A
		case 0x32: maccode = 0x17; break; // B
		case 0x21: maccode = 0x11; break; // C
		case 0x23: maccode = 0x05; break; // D
		case 0x24: maccode = 0x1D; break; // E
		case 0x2B: maccode = 0x07; break; // F
		case 0x34: maccode = 0x0B; break; // G
		case 0x33: maccode = 0x09; break; // H
		case 0x43: maccode = 0x45; break; // I
		case 0x3B: maccode = 0x4D; break; // J
		case 0x42: maccode = 0x51; break; // K
		case 0x4B: maccode = 0x4B; break; // L  // woah
		case 0x3A: maccode = 0x5D; break; // M
		case 0x31: maccode = 0x5B; break; // N
		case 0x44: maccode = 0x3F; break; // O
		case 0x4D: maccode = 0x47; break; // P
		case 0x15: maccode = 0x19; break; // Q
		case 0x2D: maccode = 0x1F; break; // R
		case 0x1B: maccode = 0x03; break; // S
		case 0x2C: maccode = 0x23; break; // T
		case 0x3C: maccode = 0x41; break; // U
		case 0x2A: maccode = 0x13; break; // V
		case 0x1D: maccode = 0x1B; break; // W
		case 0x22: maccode = 0x0F; break; // X
		case 0x35: maccode = 0x21; break; // Y
		case 0x1A: maccode = 0x0D; break; // Z
		case 0x45: maccode = 0x3B; break; // 0
		case 0x16: maccode = 0x25; break; // 1
		case 0x1E: maccode = 0x27; break; // 2
		case 0x26: maccode = 0x29; break; // 3
		case 0x25: maccode = 0x2B; break; // 4
		case 0x2E: maccode = 0x2F; break; // 5
		case 0x36: maccode = 0x2D; break; // 6
		case 0x3D: maccode = 0x35; break; // 7
		case 0x3E: maccode = 0x39; break; // 8
		case 0x46: maccode = 0x33; break; // 9
		case 0x0E: maccode = 0x65; break; // `
		case 0x4E: maccode = 0x37; break; // -
		case 0x55: maccode = 0x31; break; // =
		case 0x5D: maccode = 0x55; break; // backslash
		case 0x66: maccode = 0x67; break; // bkspc
		case 0x29: maccode = 0x63; break; // space
		case 0x0D: maccode = 0x61; break; // tab
		case 0x58: maccode = 0x73; break; // caps
		case 0x12: maccode = 0x71; break; // left shift
		case 0x59: maccode = 0x71; break; // right shift
		case 0x11: maccode = 0x75; break; // left alt = opt
		case 0x5A: maccode = 0x49; break; // enter
		case 0x41: maccode = 0x57; break; // ,
		case 0x49: maccode = 0x5F; break; // .
		case 0x4A: maccode = 0x59; break; // /
		case 0x14: maccode = 0x77; break; // L control
		case 0x76: maccode = 0x6B; break; // ESC
		case 0x52: maccode = 0x4F; break; // '
		case 0x7C: maccode = 0x79; keypad = 0x05; insertshift = 1; break; // keypad *
		case 0x7B: maccode = 0x79; keypad = 0x1D; break; // keypad -
		case 0x79: maccode = 0x79; keypad = 0x0D; insertshift = 1; break; // keypad +
		case 0x71: maccode = 0x79; keypad = 0x03; break; // keypad .
		case 0x70: maccode = 0x79; keypad = 0x25; break; // keypad 0
		case 0x69: maccode = 0x79; keypad = 0x27; break; // keypad 1
		case 0x72: maccode = 0x79; keypad = 0x29; break; // keypad 2
		case 0x7A: maccode = 0x79; keypad = 0x2B; break; // keypad 3
		case 0x6B: maccode = 0x79; keypad = 0x2D; break; // keypad 4
		case 0x73: maccode = 0x79; keypad = 0x2F; break; // keypad 5
		case 0x74: maccode = 0x79; keypad = 0x31; break; // keypad 6
		case 0x6C: maccode = 0x79; keypad = 0x33; break; // keypad 7
		case 0x75: maccode = 0x79; keypad = 0x37; break; // keypad 8
		case 0x7D: maccode = 0x79; keypad = 0x39; break; // keypad 9
		default: break;
		}
	} else {
		switch(code) {
		case 0x1F: maccode = 0x6F; break; // left windows key = cmd
		case 0x27: maccode = 0x6F; break; // right windows key = cmd
		case 0x11: maccode = 0x75; break; // right alt = cmd
		case 0x5A: maccode = 0x79; keypad = 0x19; break; // keypad enter
		case 0x4A: maccode = 0x79; keypad = 0x1B; insertshift = 1; break;  // keypad /
		case 0x75: maccode = 0x79; keypad = 0x1B; break;  // up -> keypad up
		case 0x6B: maccode = 0x79; keypad = 0x0D; break;  // left arrow
		case 0x72: maccode = 0x79; keypad = 0x11; break;  // down arrow
		case 0x74: maccode = 0x79; keypad = 0x05; break;  // right arrow
		case 0x14: maccode = 0x77; break; // R control
		}
	}

	if(keyup && maccode) {
		if(keypad) {
			keypad |= 0x80;
		} else {
			maccode |= 0x80;
		}
	}

	if(maccode == 0) maccode = 0x7B;

	__disable_irq();
	if(((key2index+1)%kBuffSize) != keyindex) {
		if((modifierkeys & kModShift) && insertshift) {
			mackey[key2index] = 0x71;
			key2index = (key2index+1) % kBuffSize;
		}
		mackey[key2index] = maccode;
		key2index = (key2index+1) % kBuffSize;
		if(keypad) {
			mackey[key2index] = keypad;
			key2index = (key2index+1) % kBuffSize;
		}
		if((modifierkeys & kModShift) && insertshift) {
			mackey[key2index] = 0xF1;
			key2index = (key2index+1) % kBuffSize;
		}
	}
	__enable_irq();

	return maccode;
}

void HandleMacCommand()
{
	switch(mackbd.cmd) {
	case 0x16:
		mackbd.state = kMacKbdStateWriteSettle;
		mackbd.delay = 10;
		mackbd.response = 0x03;
		break;
	case 0x14:
		mackbd.state = kMacKbdStateInstantWait;
		mackbd.delay = 1;
		break;
	case 0x36:
		mackbd.state = kMacKbdStateTest;
		mackbd.delay = 10;
		break;
	case 0x10:
	default:
		mackbd.state = kMacKbdStateInquiryWait;
		mackbd.delay = 25000;
		break;
	};

	return;
}
